function[F,M,trpy,drpy] = controllerAttitude(qd,obj,t,param, setpoint)
% Using these current and desired states, we have to compute the desired controls
% The current states are:
%%%%%%%%%%%% Inputs: %%%%%%%%%%%%%%%%%
%
% qd: state and desired state information for the quadrotor 
%
%  qd.pos, qd.vel   position and velocity
%  qd.euler = [roll;pitch;yaw]
%  qd.omega         angular velocity in body frame
% 
%  qd.pos_des, qd.vel_des, qd.acc_des  desired position, velocity, accel
%  qd.yaw_des, qd.yawdot_des
%
% t: current time
%    
% param: various parameters
%  param.I     moment of inertia
%  param.grav  gravitational constant g (9.8...m/s^2)
%  param.mass  mass of the drone
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Outputs: %%%%%%%%%%%%%%%%%%
%
% F: total thrust commanded (sum of forces from all rotors)
% M: total torque commanded
% trpy: thrust, roll, pitch, yaw (attitude you want to command!)
% drpy: time derivative of trpy
%% *********************PID CONTROLLER ******************************
%attitude
Kp_m = ones(3,1)*3000;
Kd_m = ones(3,1)*300;


persistent counter

if isempty(counter)
    counter = 0;
end

counter = counter + 1;

if mod(counter, 1000) == 0
    %disp(['Attitude counter: ' num2str(counter)]);
end


euler_d = setpoint(1:3);
pqr_d = [0;0;qd.yawDot_d];

%Thrust force
F = param.mass*(param.grav + setpoint(4));
%Moment force
M = param.I*(Kd_m.*(pqr_d-qd.omega)+Kp_m.*(euler_d - qd.euler));
%OUTPUT
trpy = [F, euler_d']; %thrust, roll, pitch ,yaw
drpy = [0,0,0,0];                  %time derivative of trpy
end