function[setpoint] = controllerNSB_POS(qd,obj,~,param)
% Using these current and desired states, we have to compute the desired controls
% The current states are:
%%%%%%%%%%%% Inputs: %%%%%%%%%%%%%%%%%
%
% qd: state and desired state information for the quadrotor 
%
%  qd.pos, qd.vel   position and velocity
%  qd.euler = [roll;pitch;yaw]
%  qd.omega         angular velocity in body frame
% 
%  qd.pos_des, qd.vel_des, qd.acc_des  desired position, velocity, accel
%  qd.yaw_des, qd.yawdot_des
%
% t: current time
%    
% param: various parameters
%  param.I     moment of inertia
%  param.grav  gravitational constant g (9.8...m/s^2)
%  param.mass  mass of the drone
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Outputs: %%%%%%%%%%%%%%%%%%
%
% F: total thrust commanded (sum of forces from all rotors)
% M: total torque commanded
% trpy: thrust, roll, pitch, yaw (attitude you want to command!)
% drpy: time derivative of trpy
%% *********************NSB CONTROLLER ******************************
obj = obj(1);
%wall = Box([10;20;0], [10;0;0], [10;0;20], 0.5);  
%position
Kp = [15;15;30];
Kd = [17;12;10];

persistent counter

if isempty(counter)
    counter = 0;
end

counter = counter + 1;

if mod(counter, 10) == 0
    %disp(['POS counter: ' num2str(counter)]);
end

stop = qd.pos_d;
%High level control: Anti Collision
%%%%%%%%%%%NSB%%%%%%%%%%%%%%%%
%Relative distances
    
    %OBSTACLE(S)
    
    n_1 = qd.pos(1:2) - obj.pos(1:2); %vector 2x1
    n_1(3) = 0; %making it 3x1 (only x and y matters)
    dist1 = norm(n_1);      %scalar
    
    %n2 and so on...
%     n_2 = qd.pos(1:2) - obj2.pos(1:2); %vector 2x1
%     n_2(3) = 0;
%     dist2 = norm(n_2);
%     dist_to_wall =  wall.distanceTo(qd.pos); 
%     closest_sp = wall.closestSurfacePointTo(qd.pos);
%     dist_to_wall
%     closest_sp
  
    %TASK ACTIVATION CONDITION (to release from circle)
    
    %THRESHOLD VALUES FOR ACTIVATION OF TASKS
    thresh1 = obj.thresh;
   % thresh2 = obj2.thresh;
   
   %TASK FUNCTIONS
   %1: OBSTACLEA AVOIDANCE
   sigma_1 = dist1;            %1x1
   %sigma_1d = dist_g;         %1x1
   %sigma_1d = thresh1;         %1x1
   sigma_1d = obj.radius + param.arm_length + param.rotor_radius + 0.5 ;%plot prefence);         %1x1

   %unit vector aligned with the obstacle-to-drone direction
   J1 = (n_1'/dist1);          %1x3
   J1_cross = J1'*(1/(J1*J1')); %3x1
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   lambda1 = 6; %increased to 80       %1x1     % pos def gain
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %Primary task velocity
   v_1 = (J1_cross*lambda1*(sigma_1d - sigma_1)); %3x1 STUCK (doesn't know about obstacles height)
   %Null Space contribution
   N_j1 = eye(size(J1_cross*J1,1)) - J1_cross*J1;
   
   
%    sigma_3 = dist2;            %1x1
%    %sigma_1d = dist_g;         %1x1
%    sigma_3d = thresh2;         %1x1
%    %unit vector aligned with the obstacle-to-drone direction
%    J3 = (n_2'/dist2);          %1x3
%    J3_cross = J3'*(1/(J3*J3')); %3x1
%    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    lambda1 = 6;       %1x1     % pos def gain
%    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    %Primary task velocity
%    v_3 = (J3_cross*lambda1*(sigma_3d - sigma_3)); %3x1 STUCK (doesn't know about obstacles height)
%    %Null Space contribution
%    N_j3 = eye(size(J3_cross*J3,1)) - J3_cross*J3;
   
   
   %2) REACH GOAL
   sigma_2 = qd.pos;
   sigma_2d = stop;
   %unit vector aligned with the goal-to-drone direction
   J2 = eye(size(sigma_2d,1));
   J2_cross = J2'*inv(J2*J2');
   %%%%%%%%%%%%%%%%%%%%%%%%%
   gain_L2 = 0.2; %originally 2
   Lambda2 = gain_L2*eye(size(sigma_2,1));     % pos def diagonal gain matrix 
   %%%%%%%%%%%%%%%%%%%%%%%%%
   %Secondary task velocity
   v_2 = Lambda2*(sigma_2d - sigma_2);
   %Null Space contribution
   %N_j2 = eye(size(J2_cross*J2,1)) - J2_cross*J2;
   
   %SCHEDULER 
   %theta_rad
   vec1 = stop(1:2)- qd.pos(1:2);
   vec2 = obj.pos(1:2) - qd.pos(1:2);
   vec1(3) = 0;
   vec2(3)= 0;
   theta2 = atan2(norm(cross(vec1,vec2)),dot(vec1,vec2));
   %theta2*180/pi
   theta = atan2(obj.pos(2)-qd.pos(2),obj.pos(1)-qd.pos(1));
   apo = abs(dist1*sin(theta)); %distance from 
   %abs(theta*180/pi)
   %TASK1 ACTIVSATION
   %norm(qd.vel)
       if ((dist1 < thresh1) && (apo < thresh1)) && (abs(theta2) < pi/2)
           qd.vel_d = ( v_1 +(N_j1)*v_2); % avoid obstacle
           qd.vel_d
   %TASK2 ACTIVATION
       else
                    qd.vel_d = v_2; %go towards goal
       end

%PID Low level control: Desired acceleration
% The acc_d is supposed to be given from the colav algorithm
% Position (pos_d) comes indirectly from the desired velocity (vel_d) and a time delay appears 
%acc_d = qd.acc_d + Kd.*(qd.vel_d - qd.vel) + Kp.*(qd.pos_d - qd.pos);


% NB: IN NEU
%tau_d = [0; 1; 0];
tau_d = Kd.*(qd.vel_d - qd.vel);
% Before, we caluclated tau
% to get acc_d from tau:

acc_d = tau_d / param.mass;


%roll, pitch, yaw
phi_d = sat(1/param.grav*(acc_d(1)*sin(qd.yaw_d)-acc_d(2)*cos(qd.yaw_d)),(45*pi/180));
theta_d = sat(1/param.grav*(acc_d(1)*cos(qd.yaw_d)+acc_d(2)*sin(qd.yaw_d)),(45*pi/180));
psi_d = qd.yaw_d;

euler_d = [phi_d; theta_d; psi_d];

setpoint = [euler_d; acc_d(3)];

end