function[setpoint] = controllerREACTIVE_POS(qd,obj,t,param)
% Using these current and desired states, we have to compute the desired controls
% The current states are:
%%%%%%%%%%%% Inputs: %%%%%%%%%%%%%%%%%
%
% qd: state and desired state information for the quadrotor 
%
%  qd.pos, qd.vel   position and velocity
%  qd.euler = [roll;pitch;yaw]
%  qd.omega         angular velocity in body frame
% 
%  qd.pos_des, qd.vel_des, qd.acc_des  desired position, velocity, accel
%  qd.yaw_des, qd.yawdot_des
%
% t: current time
%    
% param: various parameters
%  param.I     moment of inertia
%  param.grav  gravitational constant g (9.8...m/s^2)
%  param.mass  mass of the drone
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Outputs: %%%%%%%%%%%%%%%%%%
%
% F: total thrust commanded (sum of forces from all rotors)
% M: total torque commanded
% trpy: thrust, roll, pitch, yaw (attitude you want to command!)
% drpy: time derivative of trpy
%% *********************REACTIVE CONTROLLER ******************************
obj = obj(1:2);
%position
Kp = [15;15;30];
Kd = [17;12;10];
%REFERENCE FILTER
%qd.vel = refFilter(qd.pos_d, qd.vel_d, qd.acc_d, 3);

%High level control: Anti Collision
R = [];
sectors = zeros(6,length(obj)); %sector matrix
sensor_range = 5;
sup = [1 0 0; 0 1 0; 0 0 1]; %constants for xyz gains
%rotation matrix about Z-axis
rotangle = pi/2;
Rot = [cos(rotangle) -sin(rotangle) 0; sin(rotangle) cos(rotangle) 0; 0 0 1];
Rot2 = [cos(2*rotangle) -sin(2*rotangle) 0; sin(2*rotangle) cos(2*rotangle) 0; 0 0 1];
RotN = [cos(-rotangle) -sin(-rotangle) 0; sin(-rotangle) cos(-rotangle) 0; 0 0 1];
Rot2N = [cos(-2*rotangle) -sin(-2*rotangle) 0; sin(-2*rotangle) cos(-2*rotangle) 0; 0 0 1];
%Relative distances
dist_to_obst = [];
%rea_vel = []; %the velocity contribution from each obstacle
%alpha = ones(1, length(obj));

for i = 1:length(obj)
    R(i) = obj(i).radius;
    dist_to_obst(i) = max(norm(obj(i).pos(1:2) - qd.pos(1:2)) - R(i),0);
    if ((dist_to_obst(i) < sensor_range)) %obstacle sensed by the radar
        sector = radar_sector(qd, obj(i));
        for j = 1:6 %mark that obstacle is sensed and in which sector
            if sector == j
                sectors(j,i) =1;
            else
                sectors(j,i) = 0;
            end
        end
        %sectors
%         hold on; 
%         if t > 2.2
%             plot_all_sectors(sectors(:,i) , qd.pos, 'b')
%             %plot3(qd.pos(1) + qd.vel_d(1), qd.pos(2) + qd.vel_d(2), qd.pos_d(3), '*');
%             sectors
%             stop = 1;
%         end 
        if sectors(1, i)
            qd.vel_d =  sup*Rot*qd.vel;
        elseif sectors(2, i)
            qd.vel_d = sup*Rot2*qd.vel;
        elseif sectors(3, i)
            qd.vel_d =  qd.vel_d;
        elseif sectors(4, i)
            qd.vel_d = qd.vel_d;
        elseif sectors(5,i)
            qd.vel_d = sup*Rot2*qd.vel;
        else
            qd.vel_d = sup*Rot*qd.vel;
        end
            
        %qd.vel_d = sup*Rot*qd.vel + sup*Rot
%     else
%         rea_vel(i) = (1/length(obj))*qd.vel_d;
    end
end
%rea_vel
%qd.vel_d = alpha*rea_vel;
%PID Low level control: Desired acceleration
acc_d = qd.acc_d + Kd.*(qd.vel_d - qd.vel) + Kp.*(qd.pos_d - qd.pos);

phi_d = sat(1/param.grav*(acc_d(1)*sin(qd.yaw_d)-acc_d(2)*cos(qd.yaw_d)),(45*pi/180));
theta_d = sat(1/param.grav*(acc_d(1)*cos(qd.yaw_d)+acc_d(2)*sin(qd.yaw_d)),(45*pi/180));
psi_d = qd.yaw_d;

euler_d = [phi_d; theta_d; psi_d];

setpoint = [euler_d; acc_d(3)];     
end