%% Reference trajectory generation test
% Made by Kristian

w0 = 3;  %omega_0
xi = 1;  %zeta

ref   = 20;   %reference value x_d(step)
v_lim = 3;    %velocity limit
a_lim = 1.5;  %acceleration limit
j_lim = 3;    %jerk limit

h = 0.01;     %step size
tend = 12;
time = 0:h:tend; %time interval

x = zeros(4, length(time));   %why 4 rows? pos, vel, acc, jerk
x2 = zeros(4, length(time));

%PARAMETERS
k4_4 = (4*xi)*w0;
k3_4 = k4_4\(2 + 4*xi^2)*w0^2;
k2_4 = (k4_4*k3_4)\((4*xi)*w0^3);
k1_4 = (k4_4*k3_4*k2_4)\w0^4;



for t = 1:length(time)-1
   %for cascaded response
    p = x(1, t);
    v = x(2, t);
    a = x(3, t);
    j = x(4, t);
    %for unsaturated response
    p2 = x2(1, t);
    v2 = x2(2, t);
    a2 = x2(3, t);
    j2 = x2(4, t);
    
    % Step 1: v-controller
    tau1 = k1_4 * (ref - p);
    tau12 = k1_4 * (ref - p2); %deviation in position reference
    %saturation
    if norm(tau1) > v_lim
        tau1 = v_lim * tau1 / norm(tau1); 
    end
    
    % Step 2: a-controller
    tau2 = k2_4 * (tau1 - v);
    tau22 = k2_4 * (tau12 - v2);
    %saturation
    if norm(tau2) > a_lim
        tau2 = a_lim * tau2 / norm(tau2);
    end
    
    % Step 3: j-controller
    tau3 = k3_4 * (tau2 - a);
    tau32 = k3_4 * (tau22 - a2);
    %saturation
    if norm(tau3) > j_lim
        tau3 = j_lim * tau3 / norm(tau3);
    end
    
    % Step 4: dj controller
    tau4 = k4_4 * (tau3 - j);
    tau42 = k4_4 * (tau32 - j2);
        
    
    % Integrate
    x(:, t+1) = x(:,t) + h*[x(2:4, t); tau4];
    x2(:, t+1) = x2(:,t) + h*[x2(2:4, t); tau42];
    
    
end

figure(1); clf; hold on;
set(gca, 'ColorOrderIndex', 1);
plot(time, x(1, :), '')
plot(time, x(2, :), '');
plot(time, x(3, :), '');
%plot(time, x(4, :), '--');
legend('Position', 'Velocity', 'Acceleration', 'Jerk', 'location', 'East')

xlabel('time [s]');
title('Reference model step response')

figure(2); clf; hold on;
set(gca, 'ColorOrderIndex', 1);
plot(time, x2(1, :), '')
plot(time, x2(2, :), '');
plot(time, x2(3, :), '');
%plot(time, x2(4, :), '--');
legend('Position', 'Velocity', 'Acceleration', 'Jerk', 'location', 'East')

xlabel('time [s]');
title('Regular Response')
%a = line([0 time(end)], [v_lim v_lim]); a.Color = 'black';
%a = line([0 time(end)], [a_lim a_lim]); a.Color = 'black';


