% ***************** QUADROTOR SIMULATION *****************
close all
clear all
addpath('utils')
addpath('trajectories')
addpath('shapes')
addpath('velObst')
addpath('radar')
addpath('refFilter')


%%%%%%%%%%% TRAJECTORY GENERATOR %%%%%%%%%%%
% You can change trajectory here

trajhandle = @step;
%trajhandle = @squareAbove;
%trajhandle = @square
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%% CONTROLLER %%%%%%%%%%%%%%%%%%%%%%
% You can change the controller used here
%ATTITUDE CONTROLLER %100Hz
controlhandle = @controllerAttitude; 
%POSITION CONTROLLER %20Hz
%controlhandle_pos = @controllerPID_POS;
controlhandle_pos = @controllerNSB_POS;
%controlhandle_pos = @controllerREACTIVE_POS;
%controlhandle_pos = @controllerVO_POS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

real_time = false; %real-time 
time_tol = 300;    %max time

%parameters for simulation
param = phyfile(); %physical parameters of the drone

%Wall(s)/object(s) for simulation

%WALL
%wall = Box([0;20;0], [20;20;0], [20;20;20], 0.01);
%wall2 = Box([0;0;0], [0;20;0], [0;20;20], 0.01);

%OBJECTS
%obj = objfile(20,1,10,20,1, 2); %STANDARD
%obj = objfile(20,3,10,20,1, 2); %Avoid obst and fly into wall
obj = objfile(20,12,10,20,2, 3); 
obj2 = objfile(20,3,10,20,1, 2);
obj3 = objfile(20,1,15,20,1,2);
obj4 = objfile(20,1,5,20,1,2);
cont = contfile();
objects =  [obj obj2 obj3 obj4];

%% **************************** FIGURES *****************************
fprintf('Initializing figures...\n')
h_fig = figure;
h_3d = gca;
c_3d = gca;
object_3d = gca;
object2_3d = gca;
object3_3d =gca;
object4_3d =gca;
axis equal
grid on
view(3);
xlabel('x [m]'); ylabel('y [m]'); zlabel('z [m]')
set(gcf,'Renderer','OpenGL')

%% *********************** INITIAL CONDITIONS ***********************
fprintf('Setting initial conditions...\n')
max_iter  = 7500;      % max iteration
starttime = 0;         % start of simulation in seconds
tstep     = 0.01;      % this determines the time step at which the solution is given
cstep     = 0.05;      % image capture time interval. ALSO POS CTRL TIMESTEP!!!!
fprintf('Running pos controller with zoh at %.2f Hz \n', 1/cstep);
nstep     = cstep/tstep;
time      = starttime; % current time
err = []; % runtime errors

    % Get start and stop position
    des_start = trajhandle(0);
    des_stop  = trajhandle(inf);
    stop  = des_stop.pos;
    x0    = init_state( des_start.pos, 0 );
    xtraj = zeros(max_iter*nstep, length(x0)); %position trajectory
    ttraj = zeros(max_iter*nstep, 1);          %time trajectory

x         = x0;        % state

pos_tol   = 0.01;
vel_tol   = 0.01;

%% ************************* RUN SIMULATION *************************
%SET TO 1 IF I WANT TO SAVE VIDEO
OUTPUT_TO_VIDEO = 0;
if OUTPUT_TO_VIDEO == 1
    v = VideoWriter('trajectory.avi');
    open(v)
end

fprintf('Simulation Running....')
%%%%%%%%%%%%%%%% MAIN LOOP %%%%%%%%%%%%%%%%%%%
for iter = 1:max_iter
    iter;
    timeint = time:tstep:time+cstep;

    tic;
    
        % Initialize quad plot
        if iter == 1
                        %state, wingspan, height, max_iter, h_3d
            %wall.draw();
            %wall2.draw();
            QP = QuadPlot( x0, 0.5, 0.1,  max_iter, h_3d);
            OBJ = ObjPlot( obj.height, obj.radius, obj.pos(1),obj.pos(2), object_3d, 0);
            %OBJ2 = ObjPlot( obj2.height, obj2.radius, obj2.pos(1),obj2.pos(2), object2_3d, 0);
            %OBJ3 = ObjPlot( obj3.height, obj3.radius, obj3.pos(1),obj3.pos(2), object3_3d, 0);
            %OBJ4 = ObjPlot( obj4.height, obj4.radius, obj4.pos(1),obj4.pos(2), object4_3d, 0);
            %THRESH = ObjPlot( obj.height, obj.thresh, obj.pos(1), obj.pos(2), object_3d, 1);
            %THRESH2 = ObjPlot( obj2.height, obj2.thresh, obj2.pos(1), obj2.pos(2), object2_3d, 1);
            CONT = ContPlot(cont.length,cont.width,cont.height,c_3d);
            desired_state = trajhandle(time);
            QP.UpdateQuadPlot(x, [desired_state.pos; desired_state.vel], time);
            h_title = title(sprintf('iteration: %d, time: %4.2f', iter, time));
        end

        % Get attitude and thrust setpoints from zoh pos controller
        % convert state to quad stuct for control
        qd = stateToQd(x);
        % Get desired_state
        desired_state = trajhandle(timeint(1));

        % The desired_state is set in the trajectory generator
        qd.pos_d      = desired_state.pos;
        qd.vel_d      = desired_state.vel;
        qd.acc_d      = desired_state.acc;
        qd.yaw_d      = desired_state.yaw;
        qd.yawDot_d   = desired_state.yawDot;
        setpoint = controlhandle_pos(qd, objects, timeint(1), param);
        
            
        % Run simulation
        opts = odeset('RelTol',0.5,'AbsTol',0.5);
        [tsave, xsave] = ode45(@(t,s) quadEOM(t, s, controlhandle, trajhandle, param, objects, setpoint), timeint, x, opts);
        x    = xsave(end, :)';
        % Velocity saturation
        for state = 4:6
            x(state) = sat(x(state), param.maxV);
        end
        % Save to traj
        xtraj((iter-1)*nstep+1:iter*nstep,:) = xsave(1:end-1,:);
        ttraj((iter-1)*nstep+1:iter*nstep) = tsave(1:end-1);

        % Update quad plot
        desired_state = trajhandle(time + cstep);
        QP.UpdateQuadPlot(x, [desired_state.pos; desired_state.vel], time + cstep);
        set(h_title, 'String', sprintf('iteration: %d, time: %4.2f', iter, time + cstep))
        if OUTPUT_TO_VIDEO == 1
            im = frame2im(getframe(gcf));
            writeVideo(v,im);
        end
   
    time = time + cstep; % Update simulation time
    t = toc;
    % Check to make sure ode45 is not timing out
    if(t> cstep*50)
        err = 'Ode45 Unstable';
        break;
    end

    % Pause to make real-time
    if real_time && (t < cstep)
        pause(cstep - t);
    end

    % Check termination criteria
    if terminate_check(x, time, stop, pos_tol, vel_tol, time_tol)
        break
    end
end
%%%%%%%%% END OF MAIN LOOP %%%%%%%%%%%%%%%%%%%%%%%%%
if OUTPUT_TO_VIDEO == 1
    close(v);
end

%% ************************* POST PROCESSING *************************
% Truncate xtraj and ttraj

    xtraj = xtraj(1:iter*nstep,:);
    ttraj = ttraj(1:iter*nstep);


% Plot the saved position and velocity of the drone

    % Truncate saved variables
    QP.TruncateHist();
    % Plot position for each quad
    h_pos = figure('Name', ['Quad ' ' : position']);
    plot_state(h_pos, QP.state_hist(1:3,:), QP.time_hist, 'pos', 'vic');
    plot_state(h_pos, QP.state_des_hist(1:3,:), QP.time_hist, 'pos', 'des');
    % Plot velocity for each quad
    h_vel = figure('Name', ['Quad '  ' : velocity']);
    plot_state(h_vel, QP.state_hist(4:6,:), QP.time_hist, 'vel', 'vic');
    plot_state(h_vel, QP.state_des_hist(4:6,:), QP.time_hist, 'vel', 'des');

if(~isempty(err))
    error(err);
end
fprintf('finished.\n')
