function vo = calc_cone_VO(qd, obst)
% Calculate cone from pos tanget to a circle vehicle
    
    r = obst.thresh; % Obst radius with safety dist
    
    a_pos = [qd.pos(1); qd.pos(2)];     % Agent position
    o_pos = [obst.pos(1); obst.pos(2)];      % Obstacle position

    
    % Calc tangent points from pos to obstacle + d_safe
    P = a_pos' - o_pos';
    d2 = dot(P,P);
    Q0 = o_pos'+r^2/d2*P;
    if d2-r^2 > 0
        T = r/d2*sqrt(d2-r^2)*P*[0,1;-1,0];
        p1 = Q0-T;  % Tangent points left
        p2 = Q0+T;  % Tangent points right

        % Move apex with respect to obstacle speed
        apex = a_pos; 
        p1 = p1'; %[p1(2); p1(1)];
        p2 = p2'; %[p2(2); p2(1)];
        %apex = [apex(2), apex(1)];

        % Calc each tangent line to obstacle through apex
        ang1 = vector_ang(apex,p1);
        ang2 = vector_ang(apex,p2);
        vo = cone(apex, [ang1; ang2]); % Class to save VO 
    else
        % Corner case where calc above result in complex numbers
        ang = vector_ang(a_pos,o_pos);
        apex = a_pos;
        vo = cone(apex, [ang-1.5; ang+1.5]); % Class to save VO
    end
end

